'use strict';

const async = require('async');
const autoprefixer = require('autoprefixer');
const cons = require('consolidate');
const fs = require('fs');
const juice = require('juice');
const minify = require('html-minifier').minify;
const noop = require('lodash.noop');
const path = require('path');
const postcss = require('postcss');

/**
 * PostCSS returns a Promise-y `LazyResult`:
 * {@link https://github.com/postcss/postcss/blob/master/docs/api.md#lazyresult-class}
 */
function autoprefixCss(css, cb) {
  if (!css) {
    cb(null);
  } else {
    postcss([autoprefixer({
      browsers: 'last 4 versions',
      remove: false,
    })])
      .process(css)
      .then(result => cb(null, result.css))
      .catch(cb);
  }
}

/**
 * Add (inline) CSS to HTML.
 *
 * {@link https://www.npmjs.com/package/juice}
 *
 * @param {Object} params
 * @param {string} params.css
 * @param {string} params.html
 * @param {string} params.templatePath
 * @param {Function} cb Node-style callback
 */
function addCss({ css, html, templatePath }, cb) {
  cb(null, juice(html, {
    extraCss: css,
    preserveMediaQueries: true,
  }));
}

/**
 * Minify HTML.
 *
 * {@link https://github.com/kangax/html-minifier}
 *
 * @param {string} html
 * @param {Function} cb Node-style callback
 */
function minifyHtml(html, cb) {
  cb(null, minify(html, {
    collapseWhitespace: true,
    conservativeCollapse: true,
    minifyCSS: true,
    removeComments: true,
  }));
}

/**
 * Get getTemplate.
 *
 * @example
 * template({
 *   html: {
 *     myHtmlTemplateVar: '<h1>Hi!</h1> <p>Insert this html…</p>',
 *   },
 *   text: {
 *     myTextTemplateVar: 'Alternative!\n\nFor the text version…',
 *   },
 * }, (error, result) => {
 *   if (error) {
 *     console.error(error);
 *   } else {
 *     console.log('HTML:', result.html);
 *     console.log('Text:', result.text);
 *   }
 * });
 *
 * @param {string} [templateName=generic]
 * @param {Object} locals Local parameters to pass to Twig templates
 * @param {Object} locals.html Local parameters to apply to the HTML template
 * @param {Object} locals.text Local parameters to apply to the text template
 * @param {Function} [callback] Optional callback with Node-style arguments
 * @returns {Promise}
 */
function getTemplate(templateName, locals, callback) {
  let localCallback;
  let localLocals;
  let localTemplateName;

  if (templateName instanceof Object) {
    localTemplateName = getTemplate.DEFAULT_TEMPLATE_DIR;
    localLocals = templateName;

    if (locals instanceof Function) {
      localCallback = locals;
    }
  } else {
    localTemplateName = templateName;

    if (locals instanceof Object) {
      localLocals = locals;
    }

    if (callback instanceof Function) {
      localCallback = callback;
    }
  }

  if (!localTemplateName) {
    throw new Error('Requires a template name');
  }

  if (!localLocals) {
    throw new Error('Requires local parameters');
  } else if (!('html' in localLocals)) {
    throw new Error('Requires local parameters for HTML');
  } else if (!('text' in localLocals)) {
    throw new Error('Requires local parameters for text');
  }

  if (!localCallback) {
    localCallback = noop;
  }

  const templatePath = path.resolve(
    __dirname, '..', getTemplate.TEMPLATES_DIR, localTemplateName
  );

  return new Promise((resolve, reject) => {
    async.parallel({
      html: (cb1) => {
        async.waterfall([
          (cb1a) => {
            async.parallel({
              css: (cb1a1) => {
                async.waterfall([
                  (cb1a1a) => {
                    fs.readFile(
                      path.join(templatePath, getTemplate.CSS_FILENAME),
                      'utf-8',
                      cb1a1a
                    );
                  },
                  (css, cb1a1b) => autoprefixCss(css, cb1a1b),
                ], cb1a1);
              },
              html: (cb1a2) => {
                cons.twig(
                  path.join(templatePath, getTemplate.HTML_FILENAME),
                  localLocals.html,
                  cb1a2
                );
              },
            }, cb1a);
          },
          (results, cb1b) => {
            addCss({
              css: results.css,
              html: results.html,
              templatePath: templatePath, // eslint-disable-line object-shorthand
            }, cb1b);
          },
          (html, cb1c) => minifyHtml(html, cb1c),
        ], cb1);
      },
      text: (cb2) => {
        cons.twig(
          path.join(templatePath, getTemplate.TEXT_FILENAME),
          localLocals.text,
          cb2
        );
      },
    }, (error, results) => {
      if (error) {
        localCallback(error);
        reject(error);
      } else {
        localCallback(null, results);
        resolve(results);
      }
    });
  });
}

/**
 * Directory containing templates.
 *
 * This is useful for manipulating the path for testing.
 *
 * @property {string}
 */
getTemplate.TEMPLATES_DIR = 'templates';

/**
 * Default template directory.
 *
 * @property {string}
 */
getTemplate.DEFAULT_TEMPLATE_DIR = 'default';

/**
 * HTML filename.
 *
 * @property {string}
 */
getTemplate.HTML_FILENAME = 'html.twig';

/**
 * CSS filename.
 *
 * @property {string}
 */
getTemplate.CSS_FILENAME = 'styles.css';

/**
 * Text filename.

 * @property {string}
 */
getTemplate.TEXT_FILENAME = 'text.twig';

/**
 * Export `getTemplate` as object to make testing easier.
 */
module.exports = {
  getTemplate: getTemplate, // eslint-disable-line object-shorthand
};

